import pytest

from new.repo import complete_url

def test_inserts_default_domain_given_github_short():
    actual = complete_url('divanvisagie/new')
    assert actual == 'git@github.com:divanvisagie/new.git'

def test_does_not_insert_when_given_long():
    actual = complete_url('git@github.com:divanvisagie/new.git')
    assert actual == 'git@github.com:divanvisagie/new.git'




from os import system, chdir
import re
import shutil

def complete_url(url):
    p = re.compile('(^[A-z]*/[A-z]*)')
    if p.match(url):
        return f'git@github.com:{url}.git'
    return url

def fetch_code(project, url):
    command = f'git clone {url} {project}'
    system(command)

def ungitify(project):
    # Remove internal git from target
    shutil.rmtree(f'{project}/.git')

def reset_remotes(project):
    command = f''
    chdir(project)
    system(command)
    chdir('..')



import yaml

def get_input(prompt):
    return input(prompt)

def enter_loop_with(config, prompt=get_input):
    replacement_map = []
    for item in config['replace']['strings']:
        match = item['match']
        description = item['description']
        response = prompt(f'Enter replacement text for:\n  {match}\nDescription:\n  {description}\n > ')
        replacement_map.append((match, response))
    return replacement_map

def read(project):
    # Prompts the user for any template replacement requirements
    print(f'reading template for project: {project}')
    try:
        with open(f'{project}/.new.yml') as file:
            f = yaml.load(file, Loader=yaml.FullLoader)
            mp = enter_loop_with(f)
            return mp
    except:
        print('no template config to process')

    


#!/usr/bin/env python
import argparse
import sys
from os.path import dirname, join, abspath
from .repo import fetch_code, ungitify, complete_url
from .template import read
from .replace import replace_strings

root = abspath(join(dirname(__file__), '.')) # The root of this file

def get_options(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description='Generates new project from an existing repository')
    parser.add_argument('project', type=str, help='name of the project you want to create')
    parser.add_argument('url', type=str, help='url of the repository you want to base this project on')
    parser.add_argument('-p', '--preserve', dest='preserve', action='store_true' ,help='preserve the git history of the template')
    # parser.add_argument('-v', '--verbose', dest='verbose', action='store_true', help='display extra information such as diffs')
    # parser.add_argument('-k', '--keep', dest='keep', action='store_true', help='keep the .new.yml config file')
    return parser.parse_args(args)

def print_repo_details(project_name, git_url):
    print(f'Project name: {project_name}\nGit URL: {git_url}')

def main():
    options = get_options()
    url = complete_url(options.url)

    print_repo_details(options.project, url)
    fetch_code(options.project, url)
    if not options.preserve:
        ungitify(options.project)
    else:
        print('Preserving git history')
    stringmap = read(options.project)
    replace_strings(options.project, stringmap)

if __name__ == "__main__":
    main()



import pytest

from new.template import read, enter_loop_with

def test_given_template_user_is_prompted_twice():
    called_count = 0
    def prompt_mock(p):
        nonlocal called_count 
        called_count += 1

    test_object = {
        'replace': {
            'strings': [
                {
                    'match': 'com.divanvisagie.example', 
                    'description': 'The package name'
                }, 
                {
                    'match': 'Hello World',
                    'description': 'The string that is printed by the application'
                }
            ]
        }
    }

    enter_loop_with(test_object, prompt=prompt_mock)
    assert called_count == 2

def test_given_template_map_is_returned():
    called_count = 0
    def prompt_mock(p):
        nonlocal called_count 
        called_count += 1
        if called_count == 1:
            return 'com.example.test'
        else:
            return 'Hello Test'

    test_object = {
        'replace': {
            'strings': [
                {
                    'match': 'com.divanvisagie.example', 
                    'description': 'The package name'
                }, 
                {
                    'match': 'Hello World',
                    'description': 'The string that is printed by the application'
                }
            ]
        }
    }

    actual = enter_loop_with(test_object, prompt=prompt_mock)
    assert actual[0] == ('com.divanvisagie.example', 'com.example.test')
    assert actual[1] == ('Hello World', 'Hello Test')



import os

def get_file_paths(directory):
    paths = []
    for root, dirs, files in os.walk(directory):
        for file in files:
            p = os.path.join(root, file)
            paths.append(p)
    print(f'found paths: {paths}')
    return paths    

def replace_match(match_tuple, contents):
    (match, replacement) = match_tuple
    return str.replace(contents, match, replacement)

def replace_strings(project, strings):
    print(f'replacing ]]] {strings}')
    for current_file_path in get_file_paths(project):
        try:
            with open(current_file_path, 'r+') as f:
                # print(f'opened: {current_file_path}')
                old = f.read() # read everything in the file
                print(old)
                # print('Changed to')
                for pair in strings:
                    old = replace_match(pair, old)
                print(old)
                f.seek(0) # go to the beginning of the file
                f.write(old) # write the new line before
                f.truncate() # cut off any remainign text from the old file
        except Exception as e:
            print('Error: ', e, 'occurred.')



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='new-project', # Replace with your own username
    version='0.5.2',
    author='Divan Visagie',
    author_email='me@dvisagie.com',
    description='Generate new projects from git repositories',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/divanvisagie/new',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    entry_points = {
        'console_scripts': [
            'new = new.__main__:main'
        ]
    },
    python_requires='>=3.6',
)


import pytest

from new.replace import replace_strings, get_file_paths, replace_match

def test_get_file_paths():
    actual = get_file_paths('./tests/example_tree')
    assert './tests/example_tree/test.txt' in actual
    assert './tests/example_tree/onedeep/other_file.txt' in actual

def test_replace_match():
    test_content = 'We are the cheese cake factory'
    test_tuple = ('We','You')
    actual = replace_match(test_tuple, test_content)
    assert actual == 'You are the cheese cake factory'

    